#ifndef __SHAPE_H__
#define __SHAPE_H__

#include <iostream.h>

/** 
 * This is an abstract base class for all Shapes.
 * This class defines an interface for Shapes.  This means
 * that any Concrete subclass of Shape must implement the
 * getArea(), getPerimeter(), and getDescription() functions
 * dictated by the Shape interface.  Notice that since
 * the above functions are declared "pure virtual" that
 * we do not provide implementations in the (.cc) file.
 */
class Shape {

	public:
		/**
		 * We need to explain two things here:
	   *
		 * First, virtual means that if we ever have
		 * a pointer or a reference to a Shape and
		 * that shape happens to actually be some subclass
		 * of Shape, that we should call that subclasses' 
		 * version of the function during a function call. 
		 *
		 * Second, ending a function with "= 0" means that
		 * the function is pure virtual and that any subclass
		 * _must_ implement that function with _exactly_ the
		 * same prototype in the function arguments.
		 * In C++ the return-type is allowed to be co-varient which
		 * means that if a function:
		 *
		 * Shape& Foo(int x) = 0;
		 *
		 * is declared in Shape, then a Circle that subclasses Shape
		 * may implement it as:
		 *
		 * Circle& Foo(int x);
		 *
		 * Because Circle is a subclass  of Shape. 
		 * By making a function pure virtual,
		 * it makes a class "abstract".  This means that we
		 * cannot create a Shape on its own, but rather, only 
		 * create concrete classes that are Shapes through
		 * inheritance.
		 *  
		 * Here is an example to help you out:
		 * 
		 * Circle c;
		 * Shape& s = c; // To use the virtual functionality we must
		 *               // assign c to either a pointer or a reference
		 *               // super class 
		 * s.getArea() // calls circle's getArea() function
		 *
		 * Notice that I could not say:
		 * Shape s;
		 * because Shape is an abstract class.  I could say:
		 * Circle c;
		 * because Circle is s concrete implementation of Shape.
		 * Now we know that all Circles are Shapes, so we can say:
		 * Shape& s = c;
		 * However if we want the "virtual" behavior to be correct, n
		 * must either be a reference or a pointer.  This is because of the
		 * slicing problem (Read about it in your test p. 48).
		 * Now, when we call:
		 * s.getArea()
		 * the Circle verison of getArea() will be called.
		 * Similarly, if we wrote:
		 *
		 * Rectangle r;
		 * Shape& s = r;
		 * s.getArea();
		 *
		 * the Rectangle version of getArea() will be called.
		 */
		virtual double getArea() const = 0;
		
		/**
		 * Sub classes must implement this function
		 */
		virtual double getPerimeter() const = 0;
		
		/**
		 * Subclasses must implement this function
		 */
		virtual char* getDescription() const = 0;

};


ostream& operator<<(ostream& out, const Shape& s);

#endif // __SHAPE_H__